#include "Card.h"
#include "Timer.h"

#include <string>


//static 
bool CCard::cheat = false;

#define FLIPSTEPPROGRESS 0.01f

CCard::CCard(void)
{
	m_index = 0;			// two cards have same index to match
	m_backside = NULL;
	m_frontside = NULL;
	m_x = 0;
	m_y = 0;
	m_state = BACKSIDE;
	m_counter = 0;
	m_fire = NULL;
	m_smoke = NULL;
	m_flip_progress = 0.0f;
}

CCard::~CCard(void)
{
	if(m_fire) delete m_fire;
	if(m_smoke) delete m_smoke;
}



bool CCard::PointOnCard(int x, int y)
{
#ifdef DEBUG
	if(NULL == m_frontside) throw std::string("No frontside set to check point!");
#endif
	if((x < m_x) || (y < m_y)) return false;
	if((x >= m_x + m_backside->w) || (y >= m_y + m_frontside->h)) return false;
	return true;
}

void CCard::Update(void)
{
	m_counter++;
	switch(m_state)
	{
	case BACKSIDE:
	case FRONTSIDE:
		break;

	case FLIPTOFRONT:
		m_flip_progress += FLIPSTEPPROGRESS;
		if(m_flip_progress >= 1.0f)
		{
			m_flip_progress = 1.0f;
			m_state = FRONTSIDE;		
		}
		break;
	case FLIPTOBACK:
		m_flip_progress -= FLIPSTEPPROGRESS;
		if(m_flip_progress <= 0.0f)
		{
			m_flip_progress = 0.0f;
			m_state = BACKSIDE;			
		}
		break;
	case BURNING:
		if(m_counter >= CTimer::GetTickerBps() * 2)
		{
			m_state = SMOKING;			
			m_counter = 0;
			if(m_fire) m_fire->Stop();
			if(m_smoke) delete m_smoke;
			m_smoke = new CSmoke(m_x, m_y + m_frontside->h, m_frontside->w);
		}
		if(m_fire) m_fire->Update();
		break;
	case SMOKING:
		if(m_fire) m_fire->Update();
		if(m_smoke) 
		{
			m_smoke->Update();
			if(m_smoke->IsFinished())
			{
				delete m_smoke;
				m_smoke = NULL;
				m_state = VANISHED;
			}
		}
		else
		{
			m_state = VANISHED;
		}
		break;

	case VANISHED:
		if(m_fire) 
		{
			delete m_fire;
			m_fire = NULL;
		}
		break;
	}
}

void CCard::Draw(BITMAP* dest)
{
	switch(m_state)
	{
	case BACKSIDE:
		draw_sprite(dest, m_backside, m_x, m_y);
		break;

	case FRONTSIDE:
		draw_sprite(dest, m_frontside, m_x, m_y);
		break;

	case FLIPTOFRONT:
	case FLIPTOBACK:
		if(m_flip_progress < 0.5f)
		{
			// Show back side shrinking
			int w = (int)((float)m_backside->w * (1.0f - m_flip_progress/0.5f));
			int x = m_x + (m_backside->w - w)/2;
			masked_stretch_blit(m_backside, dest, 0, 0, m_backside->w, m_backside->h, x, m_y, w, m_backside->h);
		}
		else
		{
			// Show front side growing
			int w = (int)((float)m_frontside->w * (m_flip_progress-0.5f)/0.5f);
			int x = m_x + (m_frontside->w - w)/2;
			masked_stretch_blit(m_frontside, dest, 0, 0, m_frontside->w, m_frontside->h, x, m_y, w, m_frontside->h);
		}


		break;
	case BURNING:
		{
			int y_offset = (int)(((float)m_frontside->h) * ((float)m_counter/(float)(CTimer::GetTickerBps()*2)));
			masked_blit(m_frontside, dest, 0, y_offset, m_x, m_y + y_offset, m_frontside->w, m_frontside->h - y_offset);
			// make fire fx !!!
			if(m_fire) 
			{
				m_fire->SetPos(m_x, m_y - m_frontside->h + y_offset);
				m_fire->Draw(dest);
			}
		}
		break;
	case SMOKING:
		// card vanished, only smoke left
		if(m_fire) 
		{
			m_fire->SetPos(m_x, m_y + m_counter);
			m_fire->Draw(dest);
		}
		if(m_smoke)
		{
			m_smoke->Draw(dest);
		}
		break;

	case VANISHED:
		// Nothing to draw
		break;
	}
	if(CCard::cheat && m_state != VANISHED)
	{
		textprintf_ex(dest, font, m_x, m_y, makecol(0,0,0), -1, "%d", m_index);
	}
}

void CCard::Open(void)
{
	if(m_state != BACKSIDE) return;
	m_counter = 0;
	m_flip_progress = 0.0f;
	m_state = FLIPTOFRONT;
}

void CCard::Close(void)
{
	if(m_state != FRONTSIDE) return;
	m_counter = 0;
	m_flip_progress = 1.0f;
	m_state = FLIPTOBACK;
}

void CCard::Remove(void)
{
	if(m_state != FRONTSIDE) return;
	m_counter = 0;
	m_state = BURNING;
	if(m_fire) delete m_fire;
	m_fire = new CFire(m_x, m_y - m_frontside->h, m_frontside->w, m_frontside->h);
}
