#include "CardStack.h"

#include <stdlib.h>
#include <list>

using namespace std;


static int card_positions[NUMBER_OF_CARDS*2] = 
{
	20, 20,
	165, 20,
	310, 20,
	455, 20,

	20, 165,
	165, 165,
	310, 165,
	455, 165,

	20, 310,
	165, 310,
	310, 310,
	455, 310,

	20, 455,
	165, 455,
	310, 455,
	455, 455
};

static const char * card_front_names[NUMBER_OF_PAIRS] =
{
	"NEWS1",
	"NEWS2",
	"NEWS3",
	"NEWS4",
	"NEWS5",
	"NEWS6",
	"NEWS7",
	"NEWS8"
};



CCardStack::CCardStack(void)
{
	m_datafile = NULL;
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		m_card[i] = NULL;
	}
	m_num_open_cards = 0;
	m_open_cards[0] = -1;
	m_open_cards[1] = -1;
}


CCardStack::~CCardStack(void)
{
	DeleteAllCards();
	if(m_datafile)
	{
		unload_datafile(m_datafile);
	}
}

void CCardStack::DeleteAllCards(void)
{
	for(int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i]) delete m_card[i];
		m_card[i] = NULL;
	}
}

void CCardStack::LoadDatafile(const char * filename)
{
	m_datafile = load_datafile(filename);
	if(NULL == m_datafile) throw("Datafile could not be loaded!!!");
}


void CCardStack::CreateNewStack(void)
{
	DeleteAllCards();

	// create all cards and push them in a list
	// sorted in the list
	list<CCard*> cardlist;
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		CCard* card = new CCard;

		DATAFILE* data = find_datafile_object(m_datafile, "BACK");
		if(NULL == data) throw string("BACK not found in Datafile!");
		card->SetBackside((BITMAP*)data->dat);

		data = find_datafile_object(m_datafile, card_front_names[i/2]);
		if(NULL == data) throw string("Frontside not found in Datafile!");
		card->SetFrontside((BITMAP*)data->dat);

		card->SetIndex(i/2);

		cardlist.push_back(card);
	}
	// all cards created, so now mix them
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		// Get a random card out of the (sorted) list
		int nr = AL_RAND() % cardlist.size();
		list<CCard*>::iterator itor = cardlist.begin();
		for(int j = 0; j < nr; j++) ++itor;
		// Get the random card in the array
		m_card[i] = *itor;
		// Remove the card from the list
		cardlist.erase(itor);
		// Set the position of the card
		m_card[i]->SetPosition(card_positions[i*2], card_positions[i*2+1]);
	}
}

void CCardStack::Draw(BITMAP* dest)
{
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i]) m_card[i]->Draw(dest);
	}
}

void CCardStack::Update(void)
{
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i]) m_card[i]->Update();
	}
}

int CCardStack::GetCardIndexAtPoint(int x, int y)
{
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i])
		{
			if(m_card[i]->PointOnCard(x, y)) return i;
		}
	}
	return -1;
}

CCard* CCardStack::GetCard(int index)
{
	return m_card[index];
}

void CCardStack::CloseAllCards(void)
{
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i]) m_card[i]->Close();
	}
	m_num_open_cards = 0;
}

bool CCardStack::AreAllCardsRemoved(void)
{
	bool result = true;
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i]) result &= m_card[i]->IsRemoved();
	}
	return result;
}

bool CCardStack::AllCardsIdle(void)
{
	bool result = true;
	for (int i = 0; i < NUMBER_OF_CARDS; ++i)
	{
		if(m_card[i])
		{
			result &= m_card[i]->IsIdle();
		}
	}
	return result;
}

bool CCardStack::OpenCard(int index)
{
	if(index > NUMBER_OF_CARDS || index < 0) return false;
	if(m_num_open_cards >= 2) return false;
	if(m_card[index] == NULL) return false;
	if(m_card[index]->IsClosed())
	{
		m_card[index]->Open();
		m_open_cards[m_num_open_cards] = index;
		++m_num_open_cards;
	}
	else
	{
		return false;
	}
	return true;

}

bool CCardStack::DoOpenCardsMatch(void)
{
	if(m_num_open_cards < 2) return false;
	if(m_card[m_open_cards[0]]->GetIndex() == m_card[m_open_cards[1]]->GetIndex())
		return true;
	return false;
}

void CCardStack::RemoveOpenCards(void)
{
	if(m_num_open_cards >= 1) m_card[m_open_cards[0]]->Remove();
	if(m_num_open_cards >= 2) m_card[m_open_cards[1]]->Remove();
	m_num_open_cards = 0;
}
