#include <allegro.h>
#include "Display.h"

#include <string>
#include <sstream>
#include <iomanip>


CDisplay * Display;

void CDisplay::InitInstance(void)
{
	m_buffer[0] = NULL;
	m_buffer[1] = NULL;
	m_buffer[2] = NULL;
	m_w = 0;
	m_h = 0;
	m_depth = 0;
	m_mode = NOINIT;
	m_used_buffer = 0;
	m_screenshot_taken = false;
}


CDisplay::CDisplay(void)
{
	InitInstance();
}

CDisplay::CDisplay(MODE mode, int w, int h, int depth, int rate, bool windowed)
{
	InitInstance();
	Init(mode, w, h, depth, windowed);
}

CDisplay::~CDisplay(void)
{
	DeInit();
}

void CDisplay::DeInit(void)
{
	if(m_buffer[0]) destroy_bitmap(m_buffer[0]);
	if(m_buffer[1]) destroy_bitmap(m_buffer[1]);
	if(m_buffer[2]) destroy_bitmap(m_buffer[2]);
	set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
}

void CDisplay::Init(MODE mode, int w, int h, int depth, int rate, bool windowed)
{
	if(m_mode != NOINIT)
	{
		DeInit();
	}
	if(depth == 0) depth = desktop_color_depth();
	if(depth == 0) depth = 15;
	set_color_depth(depth);
	m_depth = depth;

	if(rate)
	{
		request_refresh_rate(rate);
	}

	bool ok = false;
	if(windowed)
	{
		if(set_gfx_mode(GFX_AUTODETECT_WINDOWED, w, h, 0, 0) == 0) ok = true;
	}
	if(!ok)
	{
		if(set_gfx_mode(GFX_AUTODETECT_FULLSCREEN, w, h, 0, 0) == 0) ok = true;
	}
	if(!ok)
	{
		throw std::string("Could not set any display mode!");
	}
	m_w = w;
	m_h = h;

	switch(mode)
	{
	case BACKBUFFER:
		// create memory buffer
		m_buffer[0] = create_bitmap(m_w, m_h);
		if(NULL == m_buffer[0]) throw std::string("Could not create backbuffer!");
		break;
	case DOUBLEBUFFER:
		// create two video buffers
		m_buffer[0] = create_video_bitmap(m_w, m_h);
		m_buffer[1] = create_video_bitmap(m_w, m_h);
		if(NULL == m_buffer[0] || NULL == m_buffer[1]) throw std::string("Could not create double buffers!");
		break;
	default:
		throw std::string("Invalid buffering method!");
	}
	m_mode = mode;
	m_used_buffer = 0;
}


void CDisplay::EndUpdate(void)
{
	switch(m_mode)
	{
	case BACKBUFFER:
		release_bitmap(m_buffer[0]);
		// blit the buffer to screen
		blit(m_buffer[0], screen, 0, 0, 0, 0, m_w, m_h);
		break;
	case DOUBLEBUFFER:
		release_bitmap(m_buffer[0]);
		// show buffer
		show_video_bitmap(m_buffer[m_used_buffer]);
		// flip to next buffer
		m_used_buffer = m_used_buffer ? 0 : 1;
		break;
	default:
		throw std::string("Invalid buffering method!");
	}
    if (key[KEY_PRTSCR])
    {
        if (!m_screenshot_taken)
        {
            SaveScreenshot(screen);
            m_screenshot_taken = true;
        }
    }
    else
    {
        m_screenshot_taken = false;
    }
}

BITMAP* CDisplay::BeginUpdate(void)
{
	acquire_bitmap(m_buffer[m_used_buffer]);
	return m_buffer[m_used_buffer];
}

void CDisplay::SaveScreenshot(BITMAP * src)
{
    int nr = 1;
    bool saved = false;
    const char * format = get_config_string("screenshot", "format", "bmp");
    while (!saved && (nr < 1000))
    {
		std::stringstream fname;
        fname << "Screenshot_";
        fname << std::setw(3) << std::setfill('0') << std::right << nr << "." << format;
        
        FILE * f = fopen(fname.str().c_str(), "r");
        if (f == NULL)
        {
            PALETTE pal;
            get_palette(pal);
            BITMAP * bmp = create_bitmap(src->w, src->h);
            if (NULL == bmp)
            {
                //Logfile->Message("Failed creating tmp-bitmap for screenshot");
                return;
            }
            blit(src, bmp, 0, 0, 0, 0, src->w, src->h);
            if (save_bitmap(fname.str().c_str(), bmp, pal))
            {
                //fname << " failed saving";
            }
            else
            {
                //fname << " saved";
            }
            //Logfile->Message(fname.str());
            destroy_bitmap(bmp);            
            saved = true;
        }
        else
        {
            fclose(f);
            nr++;
        }
    }
}

