#include "Fire.h"

#include <string>

//static
PALETTE CFire::palette;



//static 
void CFire::InitPalette(void)
{
	int c;
	/* fill our palette with a gradually altering sequence of colors */
	// this code is lend from exflame.c from the allegro examples
	// i think everyone should have this

	for (c=0; c<64; c++) {
		CFire::palette[c].r = c;
		CFire::palette[c].g = 0;
		CFire::palette[c].b = 0;
	}
	for (c=64; c<128; c++) {
		CFire::palette[c].r = 63;
		CFire::palette[c].g = c-64;
		CFire::palette[c].b = 0;
	}
	for (c=128; c<192; c++) {
		CFire::palette[c].r = 63;
		CFire::palette[c].g = 63;
		CFire::palette[c].b = c-128;
	}
	for (c=192; c<256; c++) {
		CFire::palette[c].r = 63;
		CFire::palette[c].g = 63;
		CFire::palette[c].b = 63;
	}
}


CFire::CFire(void)
{
	throw std::string("Create fire with parameters only!");
}

CFire::CFire(int x, int y, int w, int h)
{
	m_stopped = false;
	m_x = x;
	m_y = y;
	m_w = w - (w%4);
	m_h = h;
	m_temp = (unsigned char*) malloc(m_w);
	if(NULL == m_temp)
	{
		throw std::string("Could not create temp buffer for fire!");
	}
	
	m_bitmap = create_bitmap_ex(8, m_w, m_h);
	if(NULL == m_bitmap)
	{
		throw std::string("Could not create fire bitmap!");
	}
	clear_bitmap(m_bitmap);
	for (int c=0; c < FIRE_HOTSPOTS; ++c)
	{
		m_hotspot[c] = AL_RAND() % m_w;
	}
}

CFire::~CFire(void)
{
	if(m_temp) free(m_temp);
	m_temp = NULL;
}

void CFire::Update(void)
{
	int c, c2;

	acquire_bitmap(m_bitmap);
	/* zero the buffer */
	for (c = 0; c < m_w; ++c) m_temp[c] = 0;
	if(!m_stopped)
	{
		for (c=0; c < FIRE_HOTSPOTS; c++) 
		{ 
			/* display the hotspots */
			for (c2 = m_hotspot[c]-20; c2 < m_hotspot[c]+20; ++c2)
			{
				if ((c2 >= 0) && (c2 < m_w))
				{
					m_temp[c2] = MIN(m_temp[c2] + 20-abs(m_hotspot[c]-c2), 192);
				}
			}

			/* move the hotspots */
			m_hotspot[c] += (AL_RAND() & 7) - 3; 
			if (m_hotspot[c] < 20)
				m_hotspot[c] += m_w - 40;
			else
				if (m_hotspot[c] >= m_w-20)
					m_hotspot[c] -= m_w-40;
		}
	}
	/* display the buffer */
	for (c = 0; c < m_w; ++c)
		putpixel(m_bitmap, c, m_h - 1, m_temp[c]);

	for (int y = 0; y < m_h-1; ++y) 
	{
		/* read line */
		/*
		for (int x = 0; x < m_w; ++x) 
		{
			c = getpixel(m_bitmap, x, y + 1);
			if (c > 64) c-=2; else if(c > 0) --c;
			if(c < 0) c = 0;
			putpixel(m_bitmap, x, y, c);
		}
		*/

		bmp_select(m_bitmap);
		/* get an address for reading line y+1 */
		uintptr_t address = bmp_read_line(m_bitmap, y+1);
		/* read line in 32 bit chunks */
		for (int x = 0; x < m_w; x += sizeof(uint32_t))
		{
			*((uint32_t *)&m_temp[x]) = bmp_read32(address+x);
		}
		/* adjust it */
		for (int x = 0; x < m_w; ++x) 
		{
			if (m_temp[x] > 64) m_temp[x]-=2; else if(m_temp[x] > 0) --m_temp[x];
			if(m_temp[x] < 0) m_temp[x] = 0;
		}
		/* get an address for writing line y */
		address = bmp_write_line(m_bitmap, y);
		/* write line in 32 bit chunks */
		for (int x = 0; x < m_w; x += sizeof(uint32_t))
		bmp_write32(address+x, *((uint32_t *)&m_temp[x]));
		bmp_unwrite_line(m_bitmap);
	}
	release_bitmap(m_bitmap);
}

void CFire::Draw(BITMAP* dest)
{
	select_palette(CFire::palette);
	draw_sprite(dest, m_bitmap, m_x, m_y);
	unselect_palette();
}
