#include "GameScreen.h"
#include "Mouse.h"
#include "Data.h"
#include "Message.h"
#include "Display.h"


// Ticks to wait before not matching cards close
#define CLOSECARDDELAY 200

static const char txt_welcome[] = 
{
	"Welcome to >Burning-News-Memory<\n\n"
	"This is a simple memory game\n"
	"made in just one weekend\n"
	"for TINS'07 contest\n"
	"by Raabinator.\n\n"
	"-Click mouse to continue-"
};

static const char txt_rules[] = 
{
	"Instructions\n\n"
	"Rules are easy. Just match the cards\n"
	"with the same news headlines\n"
	"(from MRZ.31.2007)\n"
	"The one with the most matched cards\n"
	"wins the game.\n"
	"See readme.txt for more information\n\n"
	"Just have fun..."
};

static const char txt_quit[] = 
{
	"Thanks for playing this game.\n"
	"Visit www.raabinator.de.vu\n\n"
	"Bye bye..."
};


CGameScreen::CGameScreen(void)
{
	m_message = NULL;
	m_finished = NULL;

	m_player[0].SetInfoPos(605, 105);
	m_player[0].SetInfoSize(190, 240);
	m_player[0].SetName("Player 1");
	m_player[0].SetInfoBackColor(makecol(160,169,245));

	m_player[1].SetInfoPos(605, 355);
	m_player[1].SetInfoSize(190, 233);
	m_player[1].SetName("Player 2");
	m_player[1].SetInfoBackColor(makecol(160,169,245));

	m_bmp_pipe_h = CData::GetBitmap(PIPE_H);
	m_bmp_pipe_v = CData::GetBitmap(PIPE_V);
	m_bmp_pipe_bend_1 = CData::GetBitmap(PIPE_BEND_1);
	m_bmp_pipe_bend_2 = CData::GetBitmap(PIPE_BEND_2);
	m_bmp_pipe_bend_3 = CData::GetBitmap(PIPE_BEND_3);
	m_bmp_pipe_bend_4 = CData::GetBitmap(PIPE_BEND_4);
	m_bmp_pipe_junc_1 = CData::GetBitmap(PIPE_JUNC_1);
	m_bmp_pipe_junc_2 = CData::GetBitmap(PIPE_JUNC_2);

	m_bmp_title = CData::GetBitmap(BMP_TITLE);
	m_bmp_trashcan = CData::GetBitmap(BMP_TRASHCAN);

	m_trashcan_x = Display->W() - m_bmp_trashcan->w + 10;
	m_trashcan_y = Display->H() - m_bmp_trashcan->h + 5;


	m_state = START;
	m_act_player = -1;
}

CGameScreen::~CGameScreen(void)
{
}

bool CGameScreen::CheckTrashcanClick(void)
{
	if(!Mouse->ButtonReleased(0)) return false;
	if(Mouse->X() < m_trashcan_x) return false;
	if(Mouse->Y() < m_trashcan_y) return false;

	acquire_bitmap(m_bmp_trashcan);
	int c = getpixel(m_bmp_trashcan, Mouse->X() - m_trashcan_x, Mouse->Y() - m_trashcan_y);   
	release_bitmap(m_bmp_trashcan);

	if(c != makecol(255,0,255)) return true;
	return false;
}

void CGameScreen::Start(void)
{
	m_cardstack.LoadDatafile("Data.dat");
	m_cardstack.CreateNewStack();
}

void CGameScreen::Update(void)
{
	m_cardstack.Update();
	m_player[0].Update();
	m_player[1].Update();
	if(m_message) 
	{
		m_message->Update();
		if(m_message->Finished())
		{
			delete m_message;
			m_message = NULL;
		}
	}
	
	if (CheckTrashcanClick())	// This is the 'QUIT' button
	{
		if(m_message) delete m_message;
		m_message = new CMessage(90, 190, 420, 220, txt_quit);
		m_state = WAITQUIT;
	}

	m_counter++;

	switch(m_state)
	{
	case START:
		m_message = new CMessage(50, 180, 500, 240, txt_welcome);
		m_state = WAITSTART;
	case WAITSTART:
		if(m_message == NULL) 
		{
			m_message = new CMessage(40, 150, 520, 300, txt_rules);
			m_state = WAITRULES;
		}
		break;
	case WAITRULES:
		if(m_message == NULL) m_state = INIT;
		break;

	case INIT:
		m_cardstack.CreateNewStack();
		m_player[0].SetPairs(0);
		m_player[1].SetPairs(0);
		m_act_player = -1;
		m_state = CHANGEPLAYER;
		//break;

	case CHANGEPLAYER:
		{
			if(m_act_player >= 0) m_player[m_act_player].Highlight(false);
			m_act_player++;
			if(m_act_player > 1) m_act_player = 0;
			m_player[m_act_player].Highlight(true);
			m_state = WAITPLAYER;
			std::string s;
			s += m_player[m_act_player].GetName();
			s += "\n\n";
			s += "Your Turn Now";
			m_message = new CMessage(150, 200, 300, 200, s.c_str());
		}
	case WAITPLAYER:
		if(m_message == NULL) m_state = SELECTCARD;
		break;
	case SELECTCARD:
		if(Mouse->ButtonReleased(0))
		{
			int index = m_cardstack.GetCardIndexAtPoint(Mouse->X(), Mouse->Y());
			if(m_cardstack.OpenCard(index))
			{
				m_state = WAITCARD;
			}
		}
		break;
	case WAITCARD:
		if(m_cardstack.AllCardsIdle())
		{
			if(m_cardstack.GetNumCardsOpen() < 2)
			{
				m_state = SELECTCARD;
			}
			else
			{
				m_state = CHECKCARDS;
			}
		}
		break;
	case CHECKCARDS:
		if(m_cardstack.DoOpenCardsMatch())
		{
			m_player[m_act_player].AddPair();
			m_cardstack.RemoveOpenCards();
			m_state = WAITREMOVECARDS;
		}
		else
		{
			m_counter = 0;
			m_state = DELAYCLOSECARDS;
		}
		break;
	case DELAYCLOSECARDS:
		if(m_counter >= CLOSECARDDELAY)
		{
			m_cardstack.CloseAllCards();
			m_state = WAITCARDSCLOSED;
		}
		break;
	case WAITCARDSCLOSED:
		if(m_cardstack.AllCardsIdle()) m_state = CHANGEPLAYER;
		break;

	case WAITREMOVECARDS:
		if(m_cardstack.AllCardsIdle())
		{
			m_state = CHECKGAMEEND;
		}
		break;
	case CHECKGAMEEND:
		if(m_cardstack.AreAllCardsRemoved())
		{
			std::string s("");
			if(m_player[0].GetPairs() > m_player[1].GetPairs())
			{
				m_player[0].Highlight(true);
				m_player[1].Highlight(false);
				s += m_player[0].GetName() + "\nhas won the game!\n";
			}
			else if(m_player[1].GetPairs() > m_player[0].GetPairs())
			{
				m_player[1].Highlight(true);
				m_player[0].Highlight(false);
				s += m_player[1].GetName() + "\nhas won the game!\n";
			}
			else
			{
				m_player[0].Highlight(true);
				m_player[1].Highlight(true);
				s += "Equal pairs!\nBoth players won the game!\n";
			}
			s += "\n- Click to start a new game -";
			m_message = new CMessage(70, 190, 460, 220, s.c_str());
			m_state = PRESENTWINNER;
		}
		else
		{
			if(m_cardstack.AllCardsIdle()) m_state = SELECTCARD;
		}
		break;
	case PRESENTWINNER:
		if(m_message == NULL) m_state = INIT;
		break;
	case WAITQUIT:
		if(m_message == NULL) m_finished = true;
		break;
	}


}


void CGameScreen::Draw(BITMAP* dest)
{
	// Draw background of card area
	rectfill(dest, 0, 0, Display->W(), Display->H(), makecol(150,190,200));//makecol(203, 225, 231));
	// Draw player info
	m_player[0].DrawInfo(dest);
	m_player[1].DrawInfo(dest);

	// Draw cards
	m_cardstack.Draw(dest);

	// Draw Title bitmap
	draw_sprite(dest, m_bmp_title, 605, 5);
	
	DrawPlayerDeco(dest);

	if(m_message)
	{
		m_message->Draw(dest);
	}
	draw_sprite(dest, m_bmp_trashcan, m_trashcan_x, m_trashcan_y);
	
}

void CGameScreen::DrawPlayerDeco(BITMAP* dest)
{
	int x = 630;
	for (int i = 0; i < 7; ++i)
	{
		draw_sprite(dest, m_bmp_pipe_h, x, 0);
		draw_sprite(dest, m_bmp_pipe_h, x, 101-m_bmp_pipe_h->h/2);
		draw_sprite(dest, m_bmp_pipe_h, x, 351-m_bmp_pipe_h->h/2);
		draw_sprite(dest, m_bmp_pipe_h, x, 600-m_bmp_pipe_h->h);
		x += m_bmp_pipe_h->w;
	}
	int y = 30;
	for (int i = 0; i < 18; ++i)
	{
		draw_sprite(dest, m_bmp_pipe_v, 600, y);
		draw_sprite(dest, m_bmp_pipe_v, 800-m_bmp_pipe_v->w, y);
		y += m_bmp_pipe_v->h;
	}
	draw_sprite(dest, m_bmp_pipe_bend_1, 600, 0);
	draw_sprite(dest, m_bmp_pipe_bend_2, 800 - m_bmp_pipe_bend_2->w, 0);
	draw_sprite(dest, m_bmp_pipe_bend_3, 800 - m_bmp_pipe_bend_3->w, 600 - m_bmp_pipe_bend_3->h);
	draw_sprite(dest, m_bmp_pipe_bend_4, 600, 600 - m_bmp_pipe_bend_4->h);

	draw_sprite(dest, m_bmp_pipe_junc_1, 600 - 3, 100 - m_bmp_pipe_junc_1->h/2);
	draw_sprite(dest, m_bmp_pipe_junc_2, 800 - m_bmp_pipe_junc_2->w + 3 , 100 - m_bmp_pipe_junc_2->h/2);

	draw_sprite(dest, m_bmp_pipe_junc_1, 600 - 3, 350 - m_bmp_pipe_junc_1->h/2);
	draw_sprite(dest, m_bmp_pipe_junc_2, 800 - m_bmp_pipe_junc_2->w + 3 , 350 - m_bmp_pipe_junc_2->h/2);
}