#include "Message.h"
#include "Data.h"
#include <fblend\fblend.h>
#include "Mouse.h"

using namespace std;

CMessage::CMessage(void)
{
}

CMessage::CMessage(int x, int y, int w, int h, const char * text)
{
	m_x = x;
	m_y = y;
	m_w = w;
	m_h = h;

	m_finished = false;
	m_font = CData::GetFont(FNT_COPPER_16);
	m_text_color = makecol(0,0,0);
	m_bg_color = makecol(166,154,94);
	m_bg_intensity = 192;

	m_bmp_pipe_h = CData::GetBitmap(PIPE_H);
	m_bmp_pipe_v = CData::GetBitmap(PIPE_V);
	m_bmp_pipe_bend_1 = CData::GetBitmap(PIPE_BEND_1);
	m_bmp_pipe_bend_2 = CData::GetBitmap(PIPE_BEND_2);
	m_bmp_pipe_bend_3 = CData::GetBitmap(PIPE_BEND_3);
	m_bmp_pipe_bend_4 = CData::GetBitmap(PIPE_BEND_4);

	const char * p = text;
	while(*p != '\0')
	{
		string s("");
		while(*p != '\0' && *p != '\n')
		{
			s += *p;
			++p;
		}
		m_lines.push_back(s);
		if(*p != '\0') ++p;
	}

}


CMessage::~CMessage(void)
{
}


void CMessage::Update(void)
{
	if(Mouse->ButtonReleased(0)) m_finished = true;
}


void CMessage::Draw(BITMAP* dest)
{
	fblend_rect_trans(dest, m_x, m_y, m_w, m_h, m_bg_color, m_bg_intensity);
	int px = m_x + m_w / 2;
	int py = m_y + (m_h - m_lines.size() * text_height(m_font)) / 2;
	list<string>::iterator itor = m_lines.begin();
	while(itor != m_lines.end())
	{
		textout_centre_ex(dest, m_font, (*itor).c_str(), px, py, m_text_color, -1);
		py += text_height(m_font);
		++itor;
	}

	// horizontal pipes
	int c = (m_w-20) / m_bmp_pipe_h->w;
	int p = m_x + 10;
	int p1 = m_y - 7;
	int p2 = m_y + m_h -m_bmp_pipe_h->h + 7;
	for (int i = 0; i < c; ++i)
	{
		draw_sprite(dest, m_bmp_pipe_h, p, p1);
		draw_sprite(dest, m_bmp_pipe_h, p, p2);
		p += m_bmp_pipe_h->w;
	}
	draw_sprite(dest, m_bmp_pipe_h, m_x + m_w - m_bmp_pipe_h->w - 10, p1);
	draw_sprite(dest, m_bmp_pipe_h, m_x + m_w - m_bmp_pipe_h->w - 10, p2);

	// vertical pipes
	c = (m_h-20) / m_bmp_pipe_v->h;
	p = m_y + 10;
	p1 = m_x - 7;
	p2 = m_x + m_w - m_bmp_pipe_v->w + 7;
	for (int i = 0; i < c; ++i)
	{
		draw_sprite(dest, m_bmp_pipe_v, p1, p);
		draw_sprite(dest, m_bmp_pipe_v, p2, p);
		p += m_bmp_pipe_v->h;
	}
	draw_sprite(dest, m_bmp_pipe_v, p1, m_y + m_h - m_bmp_pipe_v->h - 10);
	draw_sprite(dest, m_bmp_pipe_v, p2, m_y + m_h - m_bmp_pipe_v->h - 10);

	// bendings
	draw_sprite(dest, m_bmp_pipe_bend_1, m_x-7, m_y-7);
	draw_sprite(dest, m_bmp_pipe_bend_2, m_x + m_w - m_bmp_pipe_bend_2->w+7, m_y-7);
	draw_sprite(dest, m_bmp_pipe_bend_3, m_x + m_w - m_bmp_pipe_bend_3->w+7, m_y + m_h - m_bmp_pipe_bend_3->h+7);
	draw_sprite(dest, m_bmp_pipe_bend_4, m_x-7, m_y + m_h - m_bmp_pipe_bend_4->h+7);

}
