#include "Smoke.h"
#include <fblend\fblend.h>
#include <math.h>


#define SPEEDVARIATION 0.0025f
#define SCALEVARIATION 0.02f
#define INTENSEVARIATION 0.0025f
#define DRIFTVARIATION 0.0125f

#define INITIALRX 10.0f
#define INITIALRY 5.0f

#define CLOUDBMPXOVERSIZING 5
#define CLOUDBMPW	50
#define CLOUDBMPH	30

#define CLOUDNOISECOUNT 256
#define CLOUDNOISEDEPTH	64

//static 
BITMAP* CCloud::cloud_bmp[CCloud::num_of_cloud_bmps];

//static 
void CCloud::InitBmp(void)
{
	for (int i = 0; i < CCloud::num_of_cloud_bmps; i++)
	{
		CCloud::cloud_bmp[i] = create_bitmap(CLOUDBMPW + CLOUDBMPXOVERSIZING, CLOUDBMPH);	// oversizing, fblend is missing 2 pixels to the right ?!?
		if(NULL == CCloud::cloud_bmp[i]) throw std::string("Failed creating Smoke Bitmap!");
		clear_to_color(CCloud::cloud_bmp[i], makecol(255,0,255));

		ellipsefill(CCloud::cloud_bmp[i], 
			CLOUDBMPW/2, 
			CLOUDBMPH/2, 
			8 + (int)(16.0f * ((float)(CCloud::num_of_cloud_bmps-i-1)/(float)CCloud::num_of_cloud_bmps)), 
			6 + (int)(8.0f * ((float)(CCloud::num_of_cloud_bmps-i-1)/(float)CCloud::num_of_cloud_bmps)), 
			makecol(32,32,32));
		// Now make some noise
		// Take random pixels and change their color/grayscale
		acquire_bitmap(CCloud::cloud_bmp[i]);
		for(int j = 0; j < CLOUDNOISECOUNT; ++j)
		{
			int x;
			int y;
			int c;
			int n;
			x = AL_RAND() % CLOUDBMPW;
			y = AL_RAND() % CLOUDBMPH;
			n = (AL_RAND() % (CLOUDNOISEDEPTH*2)) - CLOUDNOISEDEPTH;
			c = getpixel(CCloud::cloud_bmp[i], x, y);
			if (c != makecol(255,0,255))
			{
				c = makecol(
					MIN(MAX(getr(c) - n, 0), 255),
					MIN(MAX(getg(c) - n, 0), 255),
					MIN(MAX(getb(c) - n, 0), 255)
					);
				putpixel(CCloud::cloud_bmp[i], x, y, c);
			}
		}
		// Make a fuzzy outline
		for(int y = 0; y < CLOUDBMPH; ++y)
		{
			// from the left
			int x = 0;
			while (x < CLOUDBMPW/2)
			{
				int c = getpixel(CCloud::cloud_bmp[i], x, y);
				if(c != makecol(255,0,255))
				{
					int w = AL_RAND() % 4;
					while (w)
					{
						putpixel(CCloud::cloud_bmp[i], x, y, makecol(255,0,255));
						--w;
						++x;
					}
					break;	// while (x < CLOUDBMPW/2)
				}
				++x;
			}
			// from the right
			x = CLOUDBMPW-1;
			while (x > CLOUDBMPW/2)
			{
				int c = getpixel(CCloud::cloud_bmp[i], x, y);
				if(c != makecol(255,0,255))
				{
					int w = AL_RAND() % 4;
					while (w)
					{
						putpixel(CCloud::cloud_bmp[i], x, y, makecol(255,0,255));
						--w;
						--x;
					}
					break;	// while (x > CLOUDBMPW/2)
				}
				--x;
			}
		}
		release_bitmap(CCloud::cloud_bmp[i]);

	}
}

//static 
void CCloud::CleanupBmp(void)
{
	for (int i = 0;i < CCloud::num_of_cloud_bmps; ++i)
	{
		if(CCloud::cloud_bmp[i]) destroy_bitmap(CCloud::cloud_bmp[i]);
		CCloud::cloud_bmp[i] = NULL;
	}
}


CCloud::CCloud(void)
{
	throw std::string("Create cloud with parameters only!");
}

CCloud::CCloud(int x, int y, DRIFTDIR driftdir, float driftscale)
{
	// Take x and y as the middle point
	m_x = (float)x - (float)CLOUDBMPW / 2.0f;
	m_y = (float)y - (float)CLOUDBMPH / 2.0f;
	m_scale = 1.0f;
	m_intense = 0.5f;
	m_speed = 1.0f;
	m_alive = true;
	m_drift = 0.0f;
	m_driftdir = driftdir;
	m_drift_scale = driftscale;
}

CCloud::~CCloud(void)
{
}

void CCloud::Update(void)
{
	if(!m_alive) return;
	m_scale   += SCALEVARIATION;
	m_intense -= INTENSEVARIATION;
	m_speed   -= SPEEDVARIATION;
	m_drift   += DRIFTVARIATION;
	switch(m_driftdir)
	{
	case LEFT:
		m_x -= m_drift * m_drift_scale;
		break;
	case RIGHT:
		m_x += m_drift * m_drift_scale;
		break;
	}
	if(m_speed < 0.02f) m_speed = 0.02f;
	if(m_intense <= 0.0f)
	{
		m_intense = 0.0f;
		m_alive = false;
	}
	m_y -= m_speed;
}

void CCloud::Draw(BITMAP* dest)
{
	if(!m_alive) return;
	
	fblend_trans(CCloud::cloud_bmp[MIN(int(m_intense * (float)CCloud::num_of_cloud_bmps), CCloud::num_of_cloud_bmps-1)],  
		dest,  
		m_x,  
		m_y,  
		(int)(m_intense * 255.0f) 
		);  

}







CSmoke::CSmoke(void)
{
	throw std::string("Create smoke with parameters only!");
}

CSmoke::CSmoke(int x, int y, int w)
{
	m_created_clouds = 0;
	m_next_cloud_delay = 0;
	m_x = x; 
	m_y = y;
	m_w = w;

}

CSmoke::~CSmoke(void)
{
	while(m_cloud_list.size())
	{
		delete *m_cloud_list.begin();
		m_cloud_list.erase(m_cloud_list.begin());
	}
}

bool CSmoke::IsFinished()
{
	if(m_created_clouds < MAX_CLOUDS) return false;
	if(m_cloud_list.size()) return false;
	return true;
}

void CSmoke::Update(void)
{
	if(m_created_clouds < MAX_CLOUDS)
	{
		if(m_next_cloud_delay)
		{
			--m_next_cloud_delay;
		}
		else
		{
			// Let clouds only start in the middle third of the area
			int x = m_x + m_w/3;
			int w = m_w / 3;
			int px = x + AL_RAND()%w;
			float driftscale = fabs((float)(m_x + m_w/2 - px)) / (float)(m_w/2);
			CCloud::DRIFTDIR driftdir = CCloud::NONE;
			if(px < m_x + m_w/2 - 5)
			{
				// Left from the middle -> drift to left
				driftdir = CCloud::LEFT;
			}
			else if(px > m_x + m_w/2 + 5)
			{
				// Right from the middle -> drift to right
				driftdir = CCloud::RIGHT;
			}
			CCloud* cloud = new CCloud(
				px,
				m_y,
				driftdir,
				driftscale
				);
			m_cloud_list.push_back(cloud);
			m_created_clouds++;
			m_next_cloud_delay = AL_RAND() % 2;
		}
	}
	std::list<CCloud*>::iterator itor = m_cloud_list.begin();
	while(itor != m_cloud_list.end())
	{
		(*itor)->Update();
		if(!(*itor)->IsAlive())
		{
			std::list<CCloud*>::iterator next = itor;
			++next;
			delete *itor;
			m_cloud_list.erase(itor);
			itor = next;
		}
		else
		{
			++itor;
		}
	}
}

void CSmoke::Draw(BITMAP* dest)
{
	std::list<CCloud*>::iterator itor = m_cloud_list.begin();
	while(itor != m_cloud_list.end())
	{
		(*itor)->Draw(dest);
		++itor;
	}
}


