#include <allegro.h>

#include <stdlib.h>
#include <time.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include <string>

#include "Timer.h"
#include "Display.h"
#include "Mouse.h"
#include "GameScreen.h"
#include "Fire.h"
#include "Smoke.h"
#include "Data.h"

#include "Card.h"

void InitComponents(void)
{
	if(allegro_init()) 
		throw std::string("Failed initializing allegro.");
	if(install_keyboard()) 
		throw std::string("Failed installing keyboard.");
	if(install_timer()) 
		throw std::string("Failed installing timer.");
	if(install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL))
		throw std::string("Failed installing sound.");
	if(install_mouse() < 0)
		throw std::string("Failed installing mouse.");

}



int main(int argc, char *argv[])
{
	bool show_fps = false;
	bool wait_sync = false;
	bool use_doublebuffer = false;
	bool fast_exit = false;
	
	// check command line arguments
	for(int i = 0; i < argc; i++)
	{
		if(strcmp(argv[i], "cheat") == 0) CCard::cheat = true;
		if(strcmp(argv[i], "fps") == 0) show_fps = true;
		if(strcmp(argv[i], "vsync") == 0) wait_sync = true;
		if(strcmp(argv[i], "doublebuffer") == 0) use_doublebuffer = true;
		if(strcmp(argv[i], "fastexit") == 0) fast_exit = true;
	}
	
	CGameScreen* gamescreen = NULL;
	CFire* fire = NULL;
	srand ( time(NULL) );
	try
	{
		InitComponents();

		Display = new CDisplay(use_doublebuffer ? CDisplay::DOUBLEBUFFER : CDisplay::BACKBUFFER, 800, 600, 0, 60, false);
		CData::Load("gfx_obj.dat");
		Mouse = new CMouse;
		gamescreen = new CGameScreen;

		CFire::InitPalette();
		CCloud::InitBmp();

		char * text = "Hello Allegro";
		int l = ustrlen(text);

		float startpos = 0.0f;
		float s = M_PI * 2.0f / (float)l;

		CTimer::StartTicker(100);
		if(show_fps) CTimer::StartFpsTimer();
		gamescreen->Start();
		bool need_redraw = true;
		while(!(fast_exit && key[KEY_ESC]) && !gamescreen->Finished())
		{
			while(CTimer::GetTicker())
			{
				Mouse->Update();
				gamescreen->Update();
				CTimer::DecTicker();
				if(CTimer::GetTicker() > 20) CTimer::ResetTicker();
				need_redraw = true;
			}

			if(need_redraw)
			{
				need_redraw = false;
				BITMAP* buffer = Display->BeginUpdate();

				//clear_bitmap(buffer);
				gamescreen->Draw(buffer);
				Mouse->Draw(buffer);
				if(show_fps) 
				{
					textprintf_ex(buffer, font, 0, 0, makecol(255,255,255), makecol(0,0,0), "fps:%d", CTimer::GetFps());
				}
				if(wait_sync) vsync();
				Display->EndUpdate();
				if(show_fps) CTimer::AdvanceFps();
			}
		}
	}
	catch(std::string s)
	{
		allegro_message(s.c_str());
	}
	catch(std::exception e)
	{
		allegro_message(e.what());
	}
	catch(...)
	{
		allegro_message("Unknown exception!");
	}

	if(fire) delete fire;
	if(gamescreen) delete gamescreen;
	if(Mouse) delete Mouse;
	if(Display) delete Display;
	CCloud::CleanupBmp();
	CData::Unload();

	allegro_exit();
	return 0;
}
END_OF_MAIN()